/*
 * ----------------------------------------------------------------------------
 *
 *   Hatena::Question Plugin for Google Desktop Sidebar.
 *
 *  Copyright (c) 2006, Tennetiss
 *  http://www.akaiho.com/
 *
 *
 *  CHANGELOG:
 *    2006-03-20  :  initially setup
 *    2007-01-07  :  something
 *    2007-10-09  :  changed titles of each topic longer
 *
 * ----------------------------------------------------------------------------
 */

var kCurrentVersion = "1.20";
var kUpdatesUri = "http://www.akaiho.com/projects/hatenabookmarkplugin";
var kProductHomePageUri = "http://www.akaiho.com/projects/hatenabookmarkplugin";

var kHatenaUrl = "http://q.hatena.ne.jp/list?mode=rss";
var kDefaultUsername = "hotentry";
var kCheckIntervalMs;
var kMaxItems = 1000;
var kDefaultScoreForNewItem = 10000; // so new items appear at the top.
var kMaxScoreLimit = 1000000; // 


// Represents indexes inside a comma-separated value stored in options:
var kPropertyCount = 2; // how many props per url?
var kMetaDataIndex_Score = 0;
var kMetaDataIndex_Pinned = 1;

var timerId = null;
var http = null;
var contentItems = new Array();


function Main() {
  SetDefaultOptions();

  question.pinImages = new Array(
    framework.graphics.loadImage("unpinned.gif"),
    framework.graphics.loadImage("on.gif"),
    framework.graphics.loadImage("pinned.gif"));
  question.contentFlags = gddPluginFlagNone | gddContentFlagPinnable | gddContentFlagHaveDetails;
  question.maxContentItems = kMaxItems;

  pluginHelper.onAddCustomMenuItems = AddCustomMenuItems;
  pluginHelper.onShowOptionsDlg = ShowOptionsDlg;

  ResetTimeInterval(options.GetValue("Timeinterval"));
  RefreshDisplay();
}

function view_onopen(){
  Main();
}

function SetDefaultOptions() {
  options.defaultValue("Timeinterval") = "60";
}

function AddCustomMenuItems(menu) {
  menu.AddItem(refreshMenu, 0, RefreshDisplay);
}

function ResetTimeInterval(newIntervalmin){
  kCheckIntervalMs = 1000 * 60 * newIntervalmin;
  if(timerId != null) { view.clearInterval(timerId); }
  timerId = view.setInterval(RefreshDisplay, kCheckIntervalMs);
}


function RefreshDisplay() {
  var url = kHatenaUrl;
  http = new XMLHttpRequest();
  http.onreadystatechange = OnReceivedData;
  // http.setRequestHeader("If-Modified-Since", "Sat, 1 Jan 2000 00:00:00 GMT");
//  http.open("GET", url + "?noCache=" + Math.random(), true);
  http.open("GET", url, true);
  try {
    http.send();
  }
  catch(e) {
    // Just in case ... not that we expect something bad to happen here.
  }
}

function OnReceivedData() {
  if (http && http.readyState == 4) { // completed OK?
    // make array empty.
    for (var i = 0; i < contentItems.length; ++i) {
      contentItems[i] = null; // dereference each element.
    }
    contentItems.splice(0, contentItems.length);

    // Parse and add each bookmark to our array.
    var doc = new DOMDocument();
    doc.loadXML(http.responseText);

    var title, url, tags, date;
    var elem = doc.getElementsByTagName("item");
    if (elem != null && elem.length > 0) {
      for(var i = 0; i < elem.length; ++i) {
        for (var node = elem[i].firstChild; node != null;
            node = node.nextSibling) {
          if (node.nodeName == "title")
            title = node.firstChild.nodeValue;
          if (node.nodeName == "guid")
            url = node.firstChild.nodeValue;
          if (node.nodeName == "description")
            tags = node.firstChild.nodeValue;
          if (node.nodeName == "pubDate")
            date = node.firstChild.nodeValue;
        }

        if (title != null) {  // successfully parsed?
//          var questionItem = new TopicItem(title, url, tags, date);
          var questionItem = new TopicItem(tags, url, tags, date);
          contentItems.push(questionItem);
        }
      }
    }

    // http = null;
    contentItems.sort(ItemSort);
    question.removeAllContentItems();

//     utils.alert("Adding : " + contentItems.length + " items.");
    for (var i = 0; i < contentItems.length; ++i) {
      contentItems[i].AddToSidebar();
    }

  }
}

function ItemSort(a, b) {
  // if both are pinned or both are unpinned, sort by click frequency.
  // if either is pinned, that ranks higher, irrespective of click frequency.
  if (a.IsPinned()) {
    if (b.IsPinned()) {
      return a.GetScore() - b.GetScore();
    } else {
      return +1;
    }
  } else {
    if (b.IsPinned()) {
      return -1;
    } else {
      return a.GetScore() - b.GetScore();
    }
  }
}

// A class that encapsulates a topic.
function TopicItem(title, url, tags, date) {
  this.item = new ContentItem();

  this.item.layout = gddContentItemLayoutNowrapItems; // gddContentItemLayoutEmail;
  this.item.heading = title;
  this.item.source = url;
  this.item.tooltip = url + "\n" + " " + tags;
  this.item.snippet = options.GetValue("url:" + url);

  this.item.onDetailsView = OnOpenItem; // Open item even on single-click.
  this.item.onOpenItem = OnOpenItem; // And keep double-click behavior unchanged.
  this.item.onToggleItemPinnedState = ToggleItemPinnedState;

  // Initialize metadata for new topics.
  if ("" == options.GetValue("url:" + this.item.source)) {
    // Set a default score, so new bookmark shows up at the top.
    options.PutValue("url:" + this.item.source, kDefaultScoreForNewItem + ",unpinned");
  }

  // Methods.

  TopicItem.prototype.GetScore = GetScore;
  function GetScore() {
    return parseInt(GetProperty("url:" + this.item.source, kMetaDataIndex_Score));
  }

  TopicItem.prototype.IsPinned = IsPinned;
  function IsPinned() {
    return ("pinned" == GetProperty("url:" + this.item.source,
        kMetaDataIndex_Pinned));
  }

  TopicItem.prototype.AddToSidebar = AddToSidebar;
  function AddToSidebar() {
//     utils.alert(this.item.source + "\n" + options.GetValue("url:" + this.item.source));
    question.addContentItem(this.item, gddItemDisplayInSidebar);
  }

  if (this.IsPinned()) {
    this.item.flags = gddContentItemFlagNoRemove | gddContentItemFlagPinned;
  } else {
    this.item.flags = gddContentItemFlagNoRemove;
  }

  var currentScore = GetProperty("url:" + this.item.source, kMetaDataIndex_Score);
  // Must age the bookmark, else new ones will always appear at the very bottom.
  if (currentScore > 0) { // don't age below "birth".
    PutProperty("url:" + this.item.source, kMetaDataIndex_Score,
        "" + (currentScore - 1));
  }

  function OnOpenItem(item) {
    // Increment score, store it as a string.
    var currentScore = parseInt(GetProperty("url:" + item.source,
        kMetaDataIndex_Score));
    if (currentScore < kMaxScoreLimit) {
      PutProperty("url:" + item.source, kMetaDataIndex_Score, "" + (currentScore + 1));
    }

    // Since we take over the default behavior, we need to open the item ourselves.
    var wshShell = new ActiveXObject("WScript.Shell");
    wshShell.run(item.source);
    wshShell = null;
  }

  function ToggleItemPinnedState(item) {
    // Toggle pinned state.
    var wasPinned = GetProperty("url:" + item.source, kMetaDataIndex_Pinned);
    var isPinned = ("pinned" == wasPinned) ? "unpinned" : "pinned";
    PutProperty("url:" + item.source, kMetaDataIndex_Pinned, isPinned);

    // Since we take over the default behavior, we need to do the pinning ourselves.
    if ("pinned" == isPinned) {
      item.flags = gddContentItemFlagNoRemove | gddContentItemFlagPinned;
    } else {
      item.flags = gddContentItemFlagNoRemove;
    }
  }
}

function GetProperty(url, index) {
  var currValue = options.GetValue(url);
  if (currValue != "") {
    var props = currValue.split(",");
    return props[index];
  } else {
    return "";
  }
}

function PutProperty(url, index, newValue) {
  // assert (index < kPropertyCount);
  var oldValue = options.GetValue(url);
  var oldProps;
  if (oldValue != "") {
    oldProps = oldValue.split(",");
  } else {
    oldProps = new Array(kPropertyCount); // make new properties if missing.
  }
  oldProps[index] = newValue;
  options.PutValue(url, oldProps.join(","));
}

// ---- Options Dialog ----

function ShowOptionsDlg(wnd) {

  wnd.AddControl(gddWndCtrlClassLabel, 0, "", enterTimeInterval,
                 30, 110, 200, 15);
  wnd.AddControl(gddWndCtrlClassEdit, 0, "timeinterval", options.GetValue("Timeinterval"),
                 30, 125, 150, 22);
  
  wnd.AddControl(gddWndCtrlClassLabel, 0, "", resetUsageDataDesc,
                 30, 10, 300, 40);
  var resetButton = wnd.AddControl(gddWndCtrlClassButton, 0,
      "resetUsageDataButton", resetUsageDataButton,
                                   30, 60, 120, 22);
  resetButton.onClicked = OnResetButtonClicked;

  wnd.onClose = OptionsDlgClosed;
}

function OnResetButtonClicked(wnd, ctrl) {
  // Reset the scores for every item.
  for (var i = 0; i < contentItems.length; ++i) {
    PutProperty("url:" + contentItems[i].item.source, kMetaDataIndex_Score,
        "" + kDefaultScoreForNewItem);
  }
  RefreshDisplay();
}


function OptionsDlgClosed(wnd, code) {
  if (code == gddIdOK) {
    var timeinterval = wnd.GetControl("timeinterval");
    if( isNaN(timeinterval.text) || (timeinterval.text <= 0) ){
      utils.alert( inputTimeError );
    }
    else {
      options.PutValue("Timeinterval", timeinterval.text);
      ResetTimeInterval(timeinterval.text);
    }
    RefreshDisplay();
  }
}

/* From http://www.rgagnon.com/jsdetails/js-0096.html */
function encodeUrl(sStr) {
  return escape(sStr).
    replace(/\+/g, "%2B").
      replace(/\"/g, "%22").
         replace(/\'/g, "%27").
           replace(/\//g, "%2F");
}